import 'package:flutter_test/flutter_test.dart';
import 'dart:async';

import 'package:event_base/event_model.dart';

void main() {
  test('register/unregister events', () async {
  	final model = EventModel();
  	Future<String> test1() {
  		Completer<String> c = Completer();
  		model.onEvent('e1', (arg) {
	  		print(arg);
	  		c.complete('done');
	  	});
	  	model.trigger('e1', ['a1']);
	  	return c.future;
  	}
  	print(await test1());
  });
  test('register with two handler, unbind 2', () async {
  	final model = EventModel();
  	Future<String> test2() {
  		Completer<String> c = Completer();
  		var handler1 = (arg) {
	  		print(arg);
	  		c.complete('done1');
	  	};
      expect(model.events['e1'], null);
  		model.onEvent('e1', handler1);
      expect(model.events['e1'].length, 1);
  		var handler2 = (arg) {
	  		print(arg);
	  		c.complete('done2');
	  	};
  		model.onEvent('e1', handler2);
      expect(model.events['e1'].length, 2);
	  	model.offEvent('e1', handler2);
      expect(model.events['e1'].length, 1);
	  	model.trigger('e1', ['a1']);
	  	return c.future;
  	}
  	expect(await test2(), 'done1');
  });
  test('register with two handler, unbind 1', () async {
  	final model = EventModel();
  	Future<String> test2() {
  		Completer<String> c = Completer();
  		var handler1 = (arg) {
	  		print(arg);
	  		c.complete('done1');
	  	};
      expect(model.events['e1'], null);
  		model.onEvent('e1', handler1);
      expect(model.events['e1'].length, 1);
  		var handler2 = (arg) {
	  		print(arg);
	  		c.complete('done2');
	  	};
  		model.onEvent('e1', handler2);
      expect(model.events['e1'].length, 2);
	  	model.offEvent('e1', handler1);
      expect(model.events['e1'].length, 1);
	  	model.trigger('e1', ['a1']);
	  	return c.future;
  	}
  	expect(await test2(), 'done2');
  });
  test('register events at a time', () async {
    final model = EventModel();
    Future<String> test1() {
      Completer<String> c = Completer();
      model.onEvents([{
        'key': 'e1',
        'fn': (arg) {
          print(arg);
          c.complete('done');
        }
      }]);
      model.trigger('e1', ['a1']);
      return c.future;
    }
    print(await test1());
  });
  test('register events at a time, then unregister', () async {
    final model = EventModel();
    Future<String> test1() {
      Completer<String> c = Completer();
      expect(model.events['e1'], null);
      var unbindFn = model.onEvents([{
        'key': 'e1',
        'fn': (arg) {
          print(arg);
          c.complete('done');
        }
      }]);
      expect(model.events['e1'].length, 1);
      unbindFn();
      expect(model.events['e1'].length, 0);
      c.complete('done');
      return c.future;
    }
    print(await test1());
  });
}
